@@ -138,7 +138,7 @@ struct ast_bridge_channel {
 	/*! Copy of write format used by chan before join */
 	struct ast_format *write_format;
 	/*! Call ID associated with bridge channel */
-	struct ast_callid *callid;
+	ast_callid callid;
 	/*! A clone of the roles living on chan when the bridge channel joins the bridge. This may require some opacification */
 	struct bridge_roles_datastore *bridge_roles;
 	/*! Linked list information */
@@ -164,6 +164,8 @@ struct ast_bridge_channel {
 		struct timeval dtmf_tv;
 		/*! Digit currently sending into the bridge. (zero if not sending) */
 		char dtmf_digit;
+		/*! Non-zero if a T.38 session terminate is owed to the bridge. */
+		char t38_terminate;
 	} owed;
 	/*! DTMF hook sequence state */
 	struct {
@@ -172,8 +174,23 @@ struct ast_bridge_channel {
 		/*! Collected DTMF digits for DTMF hooks. */
 		char collected[MAXIMUM_DTMF_FEATURE_STRING];
 	} dtmf_hook_state;
-	/*! Non-zero if a T.38 session terminate is owed to the bridge. */
-	char owed_t38_terminate;
+	union {
+		uint32_t raw;
+		struct {
+			/*! TRUE if binaural is suspended. */
+			unsigned int binaural_suspended:1;
+			/*! TRUE if a change of binaural positions has to be performed. */
+			unsigned int binaural_pos_change:1;
+			/*! Padding */
+			unsigned int padding:30;
+		};
+	};
+	struct {
+		/*! An index mapping of where a channel's media needs to be routed */
+		struct ast_vector_int to_bridge;
+		/*! An index mapping of where a bridge's media needs to be routed */
+		struct ast_vector_int to_channel;
+	} stream_map;
 };
 
 /*!
@@ -695,6 +712,32 @@ void ast_bridge_channel_feature_digit_add(struct ast_bridge_channel *bridge_chan
  */
 void ast_bridge_channel_feature_digit(struct ast_bridge_channel *bridge_channel, int digit);
 
+/*!
+ * \brief Maps a channel's stream topology to and from the bridge
+ * \since 15.0.0
+ *
+ * \details
+ * When a channel joins a bridge or its associated stream topology is
+ * updated, each stream in the topology needs to be mapped according
+ * to its media type to the bridge.  Calling this method creates a
+ * mapping of each stream on the channel indexed to the bridge's
+ * supported media types and vice versa (i.e. bridge's media types
+ * indexed to channel streams).
+ *
+ * The first channel to join the bridge creates the initial order for
+ * the bridge's media types (e.g. a one to one mapping is made).
+ * Subsequently added channels are mapped to that order adding more
+ * media types if/when the newly added channel has more streams and/or
+ * media types specified by the bridge.
+ *
+ * \param bridge_channel Channel to map
+ *
+ * \note The bridge_channel's bridge must be locked prior to calling this function.
+ *
+ * \return Nothing
+ */
+void ast_bridge_channel_stream_map(struct ast_bridge_channel *bridge_channel);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
