@@ -129,11 +129,12 @@ struct ast_calendar {
 	int autoreminder;    /*!< If set, override any calendar_tech specific notification times and use this time (in mins) */
 	int notify_waittime; /*!< Maxiumum time to allow for a notification attempt */
 	int refresh;         /*!< When to refresh the calendar events */
+	int fetch_again_at_reload; /*!< To reload the calendar content when the module is reloaded */
 	int timeframe;       /*!< Span (in mins) of calendar data to pull with each request */
 	pthread_t thread;    /*!< The thread that the calendar is loaded/updated in */
 	ast_cond_t unload;
 	int unloading:1;
-	int pending_deletion:1; /*!< No longer used */
+	int pending_deletion:1;
 	struct ao2_container *events;  /*!< The events that are known at this time */
 };
 
