@@ -71,7 +71,7 @@ struct ast_format_interface {
 	enum ast_format_cmp_res (* const format_cmp)(const struct ast_format *format1,
 		const struct ast_format *format2);
 
-	/*! 
+	/*!
 	 * \brief Get a format with the joint compatible attributes of both provided formats.
 	 *
 	 * \param format1 The first format
@@ -270,7 +270,7 @@ int __ast_format_interface_register(const char *codec, const struct ast_format_i
  * \retval 0 success
  * \retval -1 failure
  */
-#define ast_format_interface_register(codec, interface) __ast_format_interface_register(codec, interface, ast_module_info->self)
+#define ast_format_interface_register(codec, interface) __ast_format_interface_register(codec, interface, AST_MODULE_SELF)
 
 /*!
  * \brief Get the attribute data on a format
@@ -298,6 +298,24 @@ void ast_format_set_attribute_data(struct ast_format *format, void *attribute_da
  */
 const char *ast_format_get_name(const struct ast_format *format);
 
+/*!
+ * \brief Get the channel count on a format
+ *
+ * \param The media format
+ *
+ * \return Currently set channel count
+ */
+unsigned int ast_format_get_channel_count(const struct ast_format *format);
+
+/*!
+ * \brief Set the channel count on a format
+ *
+ * \param format The media format
+ * \param channel_count The number of audio channels used
+ *
+ */
+void ast_format_set_channel_count(struct ast_format *format, unsigned int channel_count);
+
 /*!
  * \brief Get the codec associated with a format
  *
