@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/network.h"
 
 #if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__Darwin__)
@@ -174,7 +172,7 @@ static int get_local_address(struct ast_sockaddr *ourip)
 		}
 
 		bufsz = ifn.lifn_count * sizeof(struct lifreq);
-		if (!(buf = malloc(bufsz))) {
+		if (!(buf = ast_malloc(bufsz))) {
 			close(s);
 			return -1;
 		}
@@ -187,7 +185,7 @@ static int get_local_address(struct ast_sockaddr *ourip)
 		ifc.lifc_flags = 0;
 		if (ioctl(s, SIOCGLIFCONF, &ifc) < 0) {
 			close(s);
-			free(buf);
+			ast_free(buf);
 			return -1;
 		}
 
@@ -201,7 +199,7 @@ static int get_local_address(struct ast_sockaddr *ourip)
 			}
 		}
 
-		free(buf);
+		ast_free(buf);
 #endif /* SOLARIS */
 
 		close(s);
