@@ -91,16 +91,14 @@
  * $FreeBSD: src/bin/expr/expr.y,v 1.16 2000/07/22 10:59:36 se Exp $
  */
 
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
 #include <sys/cdefs.h>
 #include <sys/types.h>
 #include <stdio.h>
 
-#if !defined(STANDALONE) && !defined(STANDALONE2)	\
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-#else
+#if defined(STANDALONE) || defined(STANDALONE2)
 #ifndef __USE_ISOC99
 #define __USE_ISOC99 1
 #endif
