@@ -25,9 +25,11 @@
 	<support_level>core</support_level>
  ***/
 
-#include "asterisk.h"
+/* This reduces the size of lock structures within astobj2 objects when
+ * DEBUG_THREADS is not defined. */
+#define DEBUG_THREADS_LOOSE_ABI
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#include "asterisk.h"
 
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
@@ -50,20 +52,50 @@ static FILE *ref_log;
  * The magic number is used for consistency check.
  */
 struct __priv_data {
-	int ref_counter;
 	ao2_destructor_fn destructor_fn;
+	/*! This field is used for astobj2 and ao2_weakproxy objects to reference each other */
+	void *weakptr;
 #if defined(AO2_DEBUG)
 	/*! User data size for stats */
 	size_t data_size;
 #endif
-	/*! The ao2 object option flags */
-	uint32_t options;
-	/*! magic number.  This is used to verify that a pointer passed in is a
-	 *  valid astobj2 */
-	uint32_t magic;
+	/*! Number of references held for this object */
+	int32_t ref_counter;
+	/*!
+	 * \brief The ao2 object option flags.
+	 *
+	 * \note This field is constant after object creation.  It shares
+	 *       a uint32_t with \ref lockused and \ref magic.
+	 */
+	uint32_t options:2;
+	/*!
+	 * \brief Set to 1 when the lock is used if refdebug is enabled.
+	 *
+	 * \note This bit-field may be modified after object creation.  It
+	 *       shares a uint32_t with \ref options and \ref magic.
+	 */
+	uint32_t lockused:1;
+	/*!
+	 * \brief Magic number.
+	 *
+	 * This is used to verify that a pointer is a valid astobj2 or ao2_weak
+	 * reference.
+	 *
+	 * \note This field is constant after object creation.  It shares
+	 *       a uint32_t with \ref options and \ref lockused.
+	 *
+	 * \warning Stealing bits for any additional writable fields would cause
+	 *          reentrancy issues if using bitfields.  If any additional
+	 *          writable bits are required in the future we will need to put
+	 *          all bitfields into a single 'uint32_t flags' field and use
+	 *          atomic operations from \file lock.h to perform writes.
+	 */
+	uint32_t magic:29;
 };
 
-#define	AO2_MAGIC	0xa570b123
+#define	AO2_MAGIC	0x1a70b123
+#define	AO2_WEAK	0x1a70b122
+#define IS_AO2_MAGIC_BAD(p) (AO2_MAGIC != (p->priv_data.magic | 1))
 
 /*!
  * What an astobj2 object looks like: fixed-size private data
@@ -74,6 +106,12 @@ struct astobj2 {
 	void *user_data[0];
 };
 
+struct ao2_weakproxy_notification {
+	ao2_weakproxy_notification_cb cb;
+	void *data;
+	AST_LIST_ENTRY(ao2_weakproxy_notification) list;
+};
+
 struct ao2_lock_priv {
 	ast_mutex_t lock;
 };
@@ -98,6 +136,17 @@ struct astobj2_rwlock {
 	void *user_data[0];
 };
 
+struct ao2_lockobj_priv {
+	void *lock;
+};
+
+/* AstObj2 with locking provided by a separate object. */
+struct astobj2_lockobj {
+	struct ao2_lockobj_priv lockobj;
+	struct __priv_data priv_data;
+	void *user_data[0];
+};
+
 #ifdef AO2_DEBUG
 struct ao2_stats ao2;
 #endif
@@ -108,32 +157,31 @@ struct ao2_stats ao2;
 #define INTERNAL_OBJ_RWLOCK(user_data) \
 	((struct astobj2_rwlock *) (((char *) (user_data)) - sizeof(struct astobj2_rwlock)))
 
+#define INTERNAL_OBJ_LOCKOBJ(user_data) \
+	((struct astobj2_lockobj *) (((char *) (user_data)) - sizeof(struct astobj2_lockobj)))
+
+#define INTERNAL_OBJ(user_data) \
+	(struct astobj2 *) ((char *) user_data - sizeof(struct astobj2))
+
 /*!
  * \brief convert from a pointer _p to a user-defined object
  *
  * \return the pointer to the astobj2 structure
  */
-static struct astobj2 *INTERNAL_OBJ(void *user_data)
-{
-	struct astobj2 *p;
+#define __INTERNAL_OBJ_CHECK(user_data, file, line, func) \
+	({ \
+		struct astobj2 *p ## __LINE__; \
+		if (!user_data \
+			|| !(p ## __LINE__ = INTERNAL_OBJ(user_data)) \
+			|| IS_AO2_MAGIC_BAD(p ## __LINE__)) { \
+			log_bad_ao2(user_data, file, line, func); \
+			p ## __LINE__ = NULL; \
+		} \
+		(p ## __LINE__); \
+	})
 
-	if (!user_data) {
-		__ast_assert_failed(0, "user_data is NULL", __FILE__, __LINE__, __PRETTY_FUNCTION__);
-		return NULL;
-	}
-
-	p = (struct astobj2 *) ((char *) user_data - sizeof(*p));
-	if (AO2_MAGIC != p->priv_data.magic) {
-		char bad_magic[100];
-
-		snprintf(bad_magic, sizeof(bad_magic), "bad magic number 0x%x for object %p",
-			p->priv_data.magic, user_data);
-		__ast_assert_failed(0, bad_magic, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-		return NULL;
-	}
-
-	return p;
-}
+#define INTERNAL_OBJ_CHECK(user_data) \
+	__INTERNAL_OBJ_CHECK(user_data, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief convert from a pointer _p to an astobj2 object
@@ -142,22 +190,51 @@ static struct astobj2 *INTERNAL_OBJ(void *user_data)
  */
 #define EXTERNAL_OBJ(_p)	((_p) == NULL ? NULL : (_p)->user_data)
 
-int is_ao2_object(void *user_data)
+int internal_is_ao2_object(void *user_data)
 {
-	return (INTERNAL_OBJ(user_data) != NULL);
+	struct astobj2 *p;
+
+	if (!user_data) {
+		return 0;
+	}
+
+	p = INTERNAL_OBJ(user_data);
+
+	return !p || IS_AO2_MAGIC_BAD(p) ? 0 : 1;
+}
+
+void log_bad_ao2(void *user_data, const char *file, int line, const char *func)
+{
+	struct astobj2 *p;
+	char bad_magic[100];
+
+	if (!user_data) {
+		__ast_assert_failed(0, "user_data is NULL", file, line, func);
+		return;
+	}
+
+	p = INTERNAL_OBJ(user_data);
+	snprintf(bad_magic, sizeof(bad_magic), "bad magic number 0x%x for object %p",
+		p->priv_data.magic, user_data);
+	__ast_assert_failed(0, bad_magic, file, line, func);
 }
 
 int __ao2_lock(void *user_data, enum ao2_lock_req lock_how, const char *file, const char *func, int line, const char *var)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	int res = 0;
 
 	if (obj == NULL) {
 		return -1;
 	}
 
+	if (ref_log) {
+		obj->priv_data.lockused  = 1;
+	}
+
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
 		obj_mutex = INTERNAL_OBJ_MUTEX(user_data);
@@ -195,6 +272,10 @@ int __ao2_lock(void *user_data, enum ao2_lock_req lock_how, const char *file, co
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
 		/* The ao2 object has no lock. */
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		res = __ao2_lock(obj_lockobj->lockobj.lock, lock_how, file, func, line, var);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func, "Invalid lock option on ao2 object %p\n",
 			user_data);
@@ -206,9 +287,10 @@ int __ao2_lock(void *user_data, enum ao2_lock_req lock_how, const char *file, co
 
 int __ao2_unlock(void *user_data, const char *file, const char *func, int line, const char *var)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	int res = 0;
 	int current_value;
 
@@ -244,6 +326,10 @@ int __ao2_unlock(void *user_data, const char *file, const char *func, int line,
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
 		/* The ao2 object has no lock. */
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		res = __ao2_unlock(obj_lockobj->lockobj.lock, file, func, line, var);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func, "Invalid lock option on ao2 object %p\n",
 			user_data);
@@ -255,15 +341,20 @@ int __ao2_unlock(void *user_data, const char *file, const char *func, int line,
 
 int __ao2_trylock(void *user_data, enum ao2_lock_req lock_how, const char *file, const char *func, int line, const char *var)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	int res = 0;
 
 	if (obj == NULL) {
 		return -1;
 	}
 
+	if (ref_log) {
+		obj->priv_data.lockused  = 1;
+	}
+
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
 		obj_mutex = INTERNAL_OBJ_MUTEX(user_data);
@@ -301,13 +392,16 @@ int __ao2_trylock(void *user_data, enum ao2_lock_req lock_how, const char *file,
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
 		/* The ao2 object has no lock. */
 		return 0;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		res = __ao2_trylock(obj_lockobj->lockobj.lock, lock_how, file, func, line, var);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func, "Invalid lock option on ao2 object %p\n",
 			user_data);
 		return -1;
 	}
 
-
 	return res;
 }
 
@@ -332,6 +426,7 @@ enum ao2_lock_req __adjust_lock(void *user_data, enum ao2_lock_req lock_how, int
 {
 	struct astobj2 *obj = INTERNAL_OBJ(user_data);
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	enum ao2_lock_req orig_lock;
 
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
@@ -362,6 +457,10 @@ enum ao2_lock_req __adjust_lock(void *user_data, enum ao2_lock_req lock_how, int
 			break;
 		}
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		orig_lock = __adjust_lock(obj_lockobj->lockobj.lock, lock_how, keep_stronger);
+		break;
 	default:
 		ast_log(LOG_ERROR, "Invalid lock option on ao2 object %p\n", user_data);
 		/* Fall through */
@@ -376,9 +475,11 @@ enum ao2_lock_req __adjust_lock(void *user_data, enum ao2_lock_req lock_how, int
 
 void *ao2_object_get_lockaddr(void *user_data)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj;
 	struct astobj2_lock *obj_mutex;
 
+	obj = INTERNAL_OBJ_CHECK(user_data);
+
 	if (obj == NULL) {
 		return NULL;
 	}
@@ -394,15 +495,24 @@ void *ao2_object_get_lockaddr(void *user_data)
 	return NULL;
 }
 
-static int internal_ao2_ref(void *user_data, int delta, const char *file, int line, const char *func)
+int __ao2_ref(void *user_data, int delta,
+	const char *tag, const char *file, int line, const char *func)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
-	int current_value;
-	int ret;
+	struct astobj2_lockobj *obj_lockobj;
+	int32_t current_value;
+	int32_t ret;
+	struct ao2_weakproxy *weakproxy = NULL;
+	const char *lock_state;
 
 	if (obj == NULL) {
+		if (ref_log && user_data) {
+			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**invalid**,%s\n",
+				user_data, delta, ast_get_tid(), file, line, func, tag ?: "");
+			fflush(ref_log);
+		}
 		return -1;
 	}
 
@@ -411,14 +521,54 @@ static int internal_ao2_ref(void *user_data, int delta, const char *file, int li
 		return obj->priv_data.ref_counter;
 	}
 
+	if (delta < 0 && obj->priv_data.magic == AO2_MAGIC && (weakproxy = obj->priv_data.weakptr)) {
+		ao2_lock(weakproxy);
+	}
+
 	/* we modify with an atomic operation the reference counter */
-	ret = ast_atomic_fetchadd_int(&obj->priv_data.ref_counter, delta);
+	ret = ast_atomic_fetch_add(&obj->priv_data.ref_counter, delta, __ATOMIC_RELAXED);
 	current_value = ret + delta;
 
 #ifdef AO2_DEBUG
 	ast_atomic_fetchadd_int(&ao2.total_refs, delta);
 #endif
 
+	if (weakproxy) {
+		struct ao2_weakproxy cbs;
+
+		if (current_value == 1) {
+			/* The only remaining reference is the one owned by the weak object */
+			struct astobj2 *internal_weakproxy;
+
+			internal_weakproxy = INTERNAL_OBJ_CHECK(weakproxy);
+
+			/* Unlink the obj from the weak proxy */
+			internal_weakproxy->priv_data.weakptr = NULL;
+			obj->priv_data.weakptr = NULL;
+
+			/* transfer list to local copy so callbacks are run with weakproxy unlocked. */
+			cbs.destroyed_cb = weakproxy->destroyed_cb;
+			AST_LIST_HEAD_INIT_NOLOCK(&weakproxy->destroyed_cb);
+
+			/* weak is already unlinked from obj so this won't recurse */
+			ao2_ref(user_data, -1);
+		}
+
+		ao2_unlock(weakproxy);
+
+		if (current_value == 1) {
+			struct ao2_weakproxy_notification *destroyed_cb;
+
+			/* Notify the subscribers that weakproxy now points to NULL. */
+			while ((destroyed_cb = AST_LIST_REMOVE_HEAD(&cbs.destroyed_cb, list))) {
+				destroyed_cb->cb(weakproxy, destroyed_cb->data);
+				ast_free(destroyed_cb);
+			}
+
+			ao2_ref(weakproxy, -1);
+		}
+	}
+
 	if (0 < current_value) {
 		/* The object still lives. */
 #define EXCESSIVE_REF_COUNT		100000
@@ -429,18 +579,31 @@ static int internal_ao2_ref(void *user_data, int delta, const char *file, int li
 			/* We just reached or went over the excessive ref count trigger */
 			snprintf(excessive_ref_buf, sizeof(excessive_ref_buf),
 				"Excessive refcount %d reached on ao2 object %p",
-				current_value, user_data);
+				(int)current_value, user_data);
 			ast_log(__LOG_ERROR, file, line, func, "%s\n", excessive_ref_buf);
 
 			__ast_assert_failed(0, excessive_ref_buf, file, line, func);
 		}
+
+		if (ref_log && tag) {
+			fprintf(ref_log, "%p,%s%d,%d,%s,%d,%s,%d,%s\n", user_data,
+				(delta < 0 ? "" : "+"), delta, ast_get_tid(),
+				file, line, func, (int)ret, tag);
+			fflush(ref_log);
+		}
 		return ret;
 	}
 
 	/* this case must never happen */
 	if (current_value < 0) {
 		ast_log(__LOG_ERROR, file, line, func,
-			"Invalid refcount %d on ao2 object %p\n", current_value, user_data);
+			"Invalid refcount %d on ao2 object %p\n", (int)current_value, user_data);
+		if (ref_log) {
+			/* Log to ref_log invalid even if (tag == NULL) */
+			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**invalid**,%s\n",
+				user_data, delta, ast_get_tid(), file, line, func, tag ?: "");
+			fflush(ref_log);
+		}
 		ast_assert(0);
 		/* stop here even if assert doesn't DO_CRASH */
 		return -1;
@@ -462,66 +625,49 @@ static int internal_ao2_ref(void *user_data, int delta, const char *file, int li
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
 		obj_mutex = INTERNAL_OBJ_MUTEX(user_data);
+		lock_state = obj->priv_data.lockused ? "used" : "unused";
 		ast_mutex_destroy(&obj_mutex->mutex.lock);
 
 		ast_free(obj_mutex);
 		break;
 	case AO2_ALLOC_OPT_LOCK_RWLOCK:
 		obj_rwlock = INTERNAL_OBJ_RWLOCK(user_data);
+		lock_state = obj->priv_data.lockused ? "used" : "unused";
 		ast_rwlock_destroy(&obj_rwlock->rwlock.lock);
 
 		ast_free(obj_rwlock);
 		break;
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
+		lock_state = "none";
 		ast_free(obj);
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		lock_state = obj->priv_data.lockused ? "used" : "unused";
+		ao2_t_ref(obj_lockobj->lockobj.lock, -1, "release lockobj");
+
+		ast_free(obj_lockobj);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func,
 			"Invalid lock option on ao2 object %p\n", user_data);
+		lock_state = "invalid";
 		break;
 	}
 
+	if (ref_log && tag) {
+		fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**destructor**lock-state:%s**,%s\n",
+			user_data, delta, ast_get_tid(), file, line, func, lock_state, tag);
+		fflush(ref_log);
+	}
+
 	return ret;
 }
 
-int __ao2_ref_debug(void *user_data, int delta, const char *tag, const char *file, int line, const char *func)
-{
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
-	int old_refcount = -1;
-
-	if (obj) {
-		old_refcount = internal_ao2_ref(user_data, delta, file, line, func);
-	}
-
-	if (ref_log && user_data) {
-		if (!obj) {
-			/* Invalid object: Bad magic number. */
-			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**invalid**,%s\n",
-				user_data, delta, ast_get_tid(), file, line, func, tag);
-			fflush(ref_log);
-		} else if (old_refcount + delta == 0) {
-			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**destructor**,%s\n",
-				user_data, delta, ast_get_tid(), file, line, func, tag);
-			fflush(ref_log);
-		} else if (delta != 0) {
-			fprintf(ref_log, "%p,%s%d,%d,%s,%d,%s,%d,%s\n", user_data, (delta < 0 ? "" : "+"),
-				delta, ast_get_tid(), file, line, func, old_refcount, tag);
-			fflush(ref_log);
-		}
-	}
-
-	return old_refcount;
-}
-
-int __ao2_ref(void *user_data, int delta)
-{
-	return internal_ao2_ref(user_data, delta, __FILE__, __LINE__, __FUNCTION__);
-}
-
 void __ao2_cleanup_debug(void *obj, const char *tag, const char *file, int line, const char *function)
 {
 	if (obj) {
-		__ao2_ref_debug(obj, -1, tag, file, line, function);
+		__ao2_ref(obj, -1, tag, file, line, function);
 	}
 }
 
@@ -532,20 +678,20 @@ void __ao2_cleanup(void *obj)
 	}
 }
 
-static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options, const char *file, int line, const char *func)
+static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options,
+	void *lockobj, const char *tag, const char *file, int line, const char *func)
 {
 	/* allocation */
 	struct astobj2 *obj;
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
+	size_t overhead;
 
 	switch (options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
-#if defined(__AST_DEBUG_MALLOC)
-		obj_mutex = __ast_calloc(1, sizeof(*obj_mutex) + data_size, file, line, func);
-#else
-		obj_mutex = ast_calloc(1, sizeof(*obj_mutex) + data_size);
-#endif
+		overhead = sizeof(*obj_mutex);
+		obj_mutex = __ast_calloc(1, overhead + data_size, file, line, func);
 		if (obj_mutex == NULL) {
 			return NULL;
 		}
@@ -554,11 +700,8 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 		obj = (struct astobj2 *) &obj_mutex->priv_data;
 		break;
 	case AO2_ALLOC_OPT_LOCK_RWLOCK:
-#if defined(__AST_DEBUG_MALLOC)
-		obj_rwlock = __ast_calloc(1, sizeof(*obj_rwlock) + data_size, file, line, func);
-#else
-		obj_rwlock = ast_calloc(1, sizeof(*obj_rwlock) + data_size);
-#endif
+		overhead = sizeof(*obj_rwlock);
+		obj_rwlock = __ast_calloc(1, overhead + data_size, file, line, func);
 		if (obj_rwlock == NULL) {
 			return NULL;
 		}
@@ -567,15 +710,29 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 		obj = (struct astobj2 *) &obj_rwlock->priv_data;
 		break;
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
-#if defined(__AST_DEBUG_MALLOC)
-		obj = __ast_calloc(1, sizeof(*obj) + data_size, file, line, func);
-#else
-		obj = ast_calloc(1, sizeof(*obj) + data_size);
-#endif
+		overhead = sizeof(*obj);
+		obj = __ast_calloc(1, overhead + data_size, file, line, func);
 		if (obj == NULL) {
 			return NULL;
 		}
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		lockobj = ao2_t_bump(lockobj, "set lockobj");
+		if (!lockobj) {
+			ast_log(__LOG_ERROR, file, line, func, "AO2_ALLOC_OPT_LOCK_OBJ requires a non-NULL lockobj.\n");
+			return NULL;
+		}
+
+		overhead = sizeof(*obj_lockobj);
+		obj_lockobj = __ast_calloc(1, overhead + data_size, file, line, func);
+		if (obj_lockobj == NULL) {
+			ao2_t_ref(lockobj, -1, "release lockobj for failed alloc");
+			return NULL;
+		}
+
+		obj_lockobj->lockobj.lock = lockobj;
+		obj = (struct astobj2 *) &obj_lockobj->priv_data;
+		break;
 	default:
 		/* Invalid option value. */
 		ast_log(__LOG_DEBUG, file, line, func, "Invalid lock option requested\n");
@@ -583,8 +740,8 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 	}
 
 	/* Initialize common ao2 values. */
-	obj->priv_data.ref_counter = 1;
 	obj->priv_data.destructor_fn = destructor_fn;	/* can be NULL */
+	obj->priv_data.ref_counter = 1;
 	obj->priv_data.options = options;
 	obj->priv_data.magic = AO2_MAGIC;
 
@@ -595,47 +752,38 @@ static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_f
 	ast_atomic_fetchadd_int(&ao2.total_refs, 1);
 #endif
 
-	/* return a pointer to the user data */
-	return EXTERNAL_OBJ(obj);
-}
-
-unsigned int ao2_options_get(void *obj)
-{
-	struct astobj2 *orig_obj = INTERNAL_OBJ(obj);
-	if (!orig_obj) {
-		return 0;
-	}
-	return orig_obj->priv_data.options;
-}
-
-void *__ao2_alloc_debug(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options, const char *tag,
-	const char *file, int line, const char *func, int ref_debug)
-{
-	/* allocation */
-	void *obj;
-
-	if ((obj = internal_ao2_alloc(data_size, destructor_fn, options, file, line, func)) == NULL) {
-		return NULL;
-	}
-
-	if (ref_log) {
-		fprintf(ref_log, "%p,+1,%d,%s,%d,%s,**constructor**,%s\n", obj, ast_get_tid(), file, line, func, tag);
+	if (ref_log && tag) {
+		fprintf(ref_log, "%p,+1,%d,%s,%d,%s,**constructor**%zu**%zu**,%s\n",
+			EXTERNAL_OBJ(obj), ast_get_tid(), file, line, func, overhead, data_size, tag);
 		fflush(ref_log);
 	}
 
 	/* return a pointer to the user data */
-	return obj;
+	return EXTERNAL_OBJ(obj);
 }
 
-void *__ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options)
+void *__ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options,
+	const char *tag, const char *file, int line, const char *func)
 {
-	return internal_ao2_alloc(data_size, destructor_fn, options, __FILE__, __LINE__, __FUNCTION__);
+	return internal_ao2_alloc(data_size, destructor_fn, options, NULL, tag, file, line, func);
 }
 
-
-void __ao2_global_obj_release(struct ao2_global_obj *holder, const char *tag, const char *file, int line, const char *func, const char *name)
+void *__ao2_alloc_with_lockobj(size_t data_size, ao2_destructor_fn destructor_fn, void *lockobj,
+	const char *tag, const char *file, int line, const char *func)
 {
-	__ao2_global_obj_replace_unref(holder, NULL, tag, file, line, func, name);
+	return internal_ao2_alloc(data_size, destructor_fn, AO2_ALLOC_OPT_LOCK_OBJ, lockobj,
+		tag, file, line, func);
+}
+
+unsigned int ao2_options_get(void *obj)
+{
+	struct astobj2 *orig_obj;
+
+	orig_obj = INTERNAL_OBJ_CHECK(obj);
+	if (!orig_obj) {
+		return 0;
+	}
+	return orig_obj->priv_data.options;
 }
 
 void *__ao2_global_obj_replace(struct ao2_global_obj *holder, void *obj, const char *tag, const char *file, int line, const char *func, const char *name)
@@ -655,11 +803,7 @@ void *__ao2_global_obj_replace(struct ao2_global_obj *holder, void *obj, const c
 	}
 
 	if (obj) {
-		if (tag) {
-			__ao2_ref_debug(obj, +1, tag, file, line, func);
-		} else {
-			__ao2_ref(obj, +1);
-		}
+		__ao2_ref(obj, +1, tag, file, line, func);
 	}
 	obj_old = holder->obj;
 	holder->obj = obj;
@@ -675,11 +819,7 @@ int __ao2_global_obj_replace_unref(struct ao2_global_obj *holder, void *obj, con
 
 	obj_old = __ao2_global_obj_replace(holder, obj, tag, file, line, func, name);
 	if (obj_old) {
-		if (tag) {
-			__ao2_ref_debug(obj_old, -1, tag, file, line, func);
-		} else {
-			__ao2_ref(obj_old, -1);
-		}
+		__ao2_ref(obj_old, -1, tag, file, line, func);
 		return 1;
 	}
 	return 0;
@@ -704,11 +844,7 @@ void *__ao2_global_obj_ref(struct ao2_global_obj *holder, const char *tag, const
 
 	obj = holder->obj;
 	if (obj) {
-		if (tag) {
-			__ao2_ref_debug(obj, +1, tag, file, line, func);
-		} else {
-			__ao2_ref(obj, +1);
-		}
+		__ao2_ref(obj, +1, tag, file, line, func);
 	}
 
 	__ast_rwlock_unlock(file, line, func, &holder->lock, name);
@@ -716,6 +852,223 @@ void *__ao2_global_obj_ref(struct ao2_global_obj *holder, const char *tag, const
 	return obj;
 }
 
+
+void *__ao2_weakproxy_alloc(size_t data_size, ao2_destructor_fn destructor_fn,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct ao2_weakproxy *weakproxy;
+
+	if (data_size < sizeof(*weakproxy)) {
+		ast_assert(0);
+		ast_log(LOG_ERROR, "Requested data_size smaller than minimum.\n");
+		return NULL;
+	}
+
+	weakproxy = __ao2_alloc(data_size, destructor_fn, AO2_ALLOC_OPT_LOCK_MUTEX,
+		tag, file, line, func);
+
+	if (weakproxy) {
+		struct astobj2 *weakproxy_internal;
+
+		/* Just created weakproxy, no need to check if it's valid. */
+		weakproxy_internal = INTERNAL_OBJ(weakproxy);
+		weakproxy_internal->priv_data.magic = AO2_WEAK;
+	}
+
+	return weakproxy;
+}
+
+int __ao2_weakproxy_set_object(void *weakproxy, void *obj, int flags,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *weakproxy_internal = __INTERNAL_OBJ_CHECK(weakproxy, file, line, func);
+	struct astobj2 *obj_internal = __INTERNAL_OBJ_CHECK(obj, file, line, func);
+	int ret = -1;
+
+	if (!weakproxy_internal
+		|| weakproxy_internal->priv_data.magic != AO2_WEAK) {
+		return -1;
+	}
+
+	if (!obj_internal
+		|| obj_internal->priv_data.weakptr
+		|| obj_internal->priv_data.magic != AO2_MAGIC) {
+		return -1;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	if (!weakproxy_internal->priv_data.weakptr) {
+		__ao2_ref(obj, +1, tag, file, line, func);
+		__ao2_ref(weakproxy, +1, tag, file, line, func);
+
+		weakproxy_internal->priv_data.weakptr = obj;
+		obj_internal->priv_data.weakptr = weakproxy;
+
+		ret = 0;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+		/* It is possible for obj to be accessed now.  It's allowed
+		 * for weakproxy to already be in a container.  Another thread
+		 * could have been waiting for a lock on weakproxy to retrieve
+		 * the object.
+		 */
+	}
+
+	return ret;
+}
+
+int __ao2_weakproxy_ref_object(void *weakproxy, int delta, int flags,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *internal = __INTERNAL_OBJ_CHECK(weakproxy, file, line, func);
+	int ret = -1;
+
+	if (!internal || internal->priv_data.magic != AO2_WEAK) {
+		/* This method is meant to be run on weakproxy objects! */
+		return -2;
+	}
+
+	/* We have a weak object, grab lock. */
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	if (internal->priv_data.weakptr) {
+		ret = __ao2_ref(internal->priv_data.weakptr, delta, tag, file, line, func);
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	return ret;
+}
+
+void *__ao2_weakproxy_get_object(void *weakproxy, int flags,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *internal = __INTERNAL_OBJ_CHECK(weakproxy, file, line, func);
+	void *obj;
+
+	if (!internal || internal->priv_data.magic != AO2_WEAK) {
+		/* This method is meant to be run on weakproxy objects! */
+		return NULL;
+	}
+
+	/* We have a weak object, grab reference to object within lock */
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	obj = internal->priv_data.weakptr;
+	if (obj) {
+		__ao2_ref(obj, +1, tag, file, line, func);
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	return obj;
+}
+
+void *__ao2_get_weakproxy(void *obj, const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *obj_internal = __INTERNAL_OBJ_CHECK(obj, file, line, func);
+
+	if (!obj_internal || obj_internal->priv_data.magic != AO2_MAGIC) {
+		/* This method is meant to be run on normal ao2 objects! */
+		return NULL;
+	}
+
+	if (!obj_internal->priv_data.weakptr) {
+		return NULL;
+	}
+
+	__ao2_ref(obj_internal->priv_data.weakptr, +1, tag, file, line, func);
+	return obj_internal->priv_data.weakptr;
+}
+
+int ao2_weakproxy_subscribe(void *weakproxy, ao2_weakproxy_notification_cb cb, void *data, int flags)
+{
+	struct astobj2 *weakproxy_internal = INTERNAL_OBJ_CHECK(weakproxy);
+	int ret = -1;
+	int hasobj;
+
+	if (!weakproxy_internal || weakproxy_internal->priv_data.magic != AO2_WEAK) {
+		return -1;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	hasobj = weakproxy_internal->priv_data.weakptr != NULL;
+	if (hasobj) {
+		struct ao2_weakproxy *weak = weakproxy;
+		struct ao2_weakproxy_notification *sub = ast_calloc(1, sizeof(*sub));
+
+		if (sub) {
+			sub->cb = cb;
+			sub->data = data;
+			AST_LIST_INSERT_HEAD(&weak->destroyed_cb, sub, list);
+			ret = 0;
+		}
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	if (!hasobj) {
+		cb(weakproxy, data);
+		ret = 0;
+	}
+
+	return ret;
+}
+
+int ao2_weakproxy_unsubscribe(void *weakproxy, ao2_weakproxy_notification_cb destroyed_cb, void *data, int flags)
+{
+	struct astobj2 *internal_weakproxy = INTERNAL_OBJ_CHECK(weakproxy);
+	struct ao2_weakproxy *weak;
+	struct ao2_weakproxy_notification *sub;
+	int ret = 0;
+
+	if (!internal_weakproxy || internal_weakproxy->priv_data.magic != AO2_WEAK || !destroyed_cb) {
+		return -1;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	weak = weakproxy;
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&weak->destroyed_cb, sub, list) {
+		if (sub->cb == destroyed_cb && sub->data == data) {
+			AST_LIST_REMOVE_CURRENT(list);
+			ast_free(sub);
+			ret++;
+			if (!(flags & OBJ_MULTIPLE)) {
+				break;
+			}
+		}
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	return ret;
+}
+
+
 #ifdef AO2_DEBUG
 static int print_cb(void *obj, void *arg, int flag)
 {
@@ -869,26 +1222,30 @@ static void astobj2_cleanup(void)
 #if defined(AO2_DEBUG)
 	ast_cli_unregister_multiple(cli_astobj2, ARRAY_LEN(cli_astobj2));
 #endif
-#ifdef REF_DEBUG
-	fclose(ref_log);
-	ref_log = NULL;
-#endif
+
+	if (ast_opt_ref_debug) {
+		fclose(ref_log);
+		ref_log = NULL;
+	}
 }
 
 int astobj2_init(void)
 {
-#ifdef REF_DEBUG
 	char ref_filename[1024];
 
-	snprintf(ref_filename, sizeof(ref_filename), "%s/refs", ast_config_AST_LOG_DIR);
-	ref_log = fopen(ref_filename, "w");
-	if (!ref_log) {
-		ast_log(LOG_ERROR, "Could not open ref debug log file: %s\n", ref_filename);
+	if (ast_opt_ref_debug) {
+		snprintf(ref_filename, sizeof(ref_filename), "%s/refs", ast_config_AST_LOG_DIR);
+		ref_log = fopen(ref_filename, "w");
+		if (!ref_log) {
+			ast_log(LOG_ERROR, "Could not open ref debug log file: %s\n", ref_filename);
+		}
 	}
-#endif
+
+	ast_register_cleanup(astobj2_cleanup);
 
 	if (container_init() != 0) {
 		fclose(ref_log);
+		ref_log = NULL;
 		return -1;
 	}
 
@@ -896,7 +1253,5 @@ int astobj2_init(void)
 	ast_cli_register_multiple(cli_astobj2, ARRAY_LEN(cli_astobj2));
 #endif	/* defined(AO2_DEBUG) */
 
-	ast_register_cleanup(astobj2_cleanup);
-
 	return 0;
 }
