@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/causes.h"
 #include "asterisk/channel.h"
 #include "asterisk/stasis_channels.h"
@@ -325,6 +323,20 @@ int ast_unreal_write(struct ast_channel *ast, struct ast_frame *f)
 		return -1;
 	}
 
+	/* If we are told to write a frame with a type that has no corresponding
+	 * stream on the channel then drop it.
+	 */
+	if (f->frametype == AST_FRAME_VOICE) {
+		if (!ast_channel_get_default_stream(ast, AST_MEDIA_TYPE_AUDIO)) {
+			return 0;
+		}
+	} else if (f->frametype == AST_FRAME_VIDEO ||
+		(f->frametype == AST_FRAME_CONTROL && f->subclass.integer == AST_CONTROL_VIDUPDATE)) {
+		if (!ast_channel_get_default_stream(ast, AST_MEDIA_TYPE_VIDEO)) {
+			return 0;
+		}
+	}
+
 	/* Just queue for delivery to the other side */
 	ao2_ref(p, 1);
 	ao2_lock(p);
@@ -734,12 +746,11 @@ int ast_unreal_channel_push_to_bridge(struct ast_channel *ast, struct ast_bridge
 	struct ast_bridge_features *features;
 	struct ast_channel *chan;
 	struct ast_channel *owner;
+	ast_callid bridge_callid;
 	RAII_VAR(struct ast_unreal_pvt *, p, NULL, ao2_cleanup);
 
-	RAII_VAR(struct ast_callid *, bridge_callid, NULL, ast_callid_cleanup);
-
 	ast_bridge_lock(bridge);
-	bridge_callid = bridge->callid ? ast_callid_ref(bridge->callid) : NULL;
+	bridge_callid = bridge->callid;
 	ast_bridge_unlock(bridge);
 
 	{
@@ -768,8 +779,8 @@ int ast_unreal_channel_push_to_bridge(struct ast_channel *ast, struct ast_bridge
 	}
 
 	if (bridge_callid) {
-		struct ast_callid *chan_callid;
-		struct ast_callid *owner_callid;
+		ast_callid chan_callid;
+		ast_callid owner_callid;
 
 		/* chan side call ID setting */
 		ast_channel_lock(chan);
@@ -779,7 +790,6 @@ int ast_unreal_channel_push_to_bridge(struct ast_channel *ast, struct ast_bridge
 			ast_channel_callid_set(chan, bridge_callid);
 		}
 		ast_channel_unlock(chan);
-		ast_callid_cleanup(chan_callid);
 
 		/* owner side call ID setting */
 		ast_channel_lock(owner);
@@ -790,7 +800,6 @@ int ast_unreal_channel_push_to_bridge(struct ast_channel *ast, struct ast_bridge
 		}
 
 		ast_channel_unlock(owner);
-		ast_callid_cleanup(owner_callid);
 	}
 
 	/* We are done with the owner now that its call ID matches the bridge */
@@ -941,7 +950,7 @@ struct ast_unreal_pvt *ast_unreal_alloc(size_t size, ao2_destructor_fn destructo
 struct ast_channel *ast_unreal_new_channels(struct ast_unreal_pvt *p,
 	const struct ast_channel_tech *tech, int semi1_state, int semi2_state,
 	const char *exten, const char *context, const struct ast_assigned_ids *assignedids,
-	const struct ast_channel *requestor, struct ast_callid *callid)
+	const struct ast_channel *requestor, ast_callid callid)
 {
 	struct ast_channel *owner;
 	struct ast_channel *chan;
