@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/sorcery.h"
 #include "asterisk/astobj2.h"
@@ -129,6 +127,9 @@ struct ast_sorcery_object {
 
 	/*! \brief Extended object fields */
 	struct ast_variable *extended;
+
+	/*! \brief Time that the object was created */
+	struct timeval created;
 };
 
 /*! \brief Structure for registered object type */
@@ -206,6 +207,13 @@ struct ast_sorcery_object_field {
 	intptr_t args[];
 };
 
+/*! \brief Proxy object for sorcery */
+struct sorcery_proxy {
+	AO2_WEAKPROXY();
+	/*! \brief The name of the module owning this sorcery instance */
+	char module_name[0];
+};
+
 /*! \brief Full structure for sorcery */
 struct ast_sorcery {
 	/*! \brief Container for known object types */
@@ -214,8 +222,8 @@ struct ast_sorcery {
 	/*! \brief Observers */
 	struct ao2_container *observers;
 
-	/*! \brief The name of the module owning this sorcery instance */
-	char module_name[0];
+	/*! \brief Pointer to module_name in the associated sorcery_proxy. */
+	char *module_name;
 };
 
 /*! \brief Structure for passing load/reload details */
@@ -335,13 +343,10 @@ static sorcery_field_handler sorcery_field_default_handler(enum aco_option_type
 	return NULL;
 }
 
-/*! \brief Hashing function for sorcery wizards */
+/*! \brief Hashing and comparison functions for sorcery wizards */
 AO2_STRING_FIELD_HASH_FN(ast_sorcery_internal_wizard, callbacks.name)
-
-/*! \brief Comparator function for sorcery wizards */
 AO2_STRING_FIELD_CMP_FN(ast_sorcery_internal_wizard, callbacks.name)
 
-/*! \brief Hashing function for sorcery wizards */
 AO2_STRING_FIELD_HASH_FN(ast_sorcery_object_field, name)
 AO2_STRING_FIELD_CMP_FN(ast_sorcery_object_field, name)
 
@@ -359,10 +364,10 @@ static void sorcery_cleanup(void)
 }
 
 /*! \brief Compare function for sorcery instances */
-AO2_STRING_FIELD_CMP_FN(ast_sorcery, module_name)
+AO2_STRING_FIELD_CMP_FN(sorcery_proxy, module_name)
 
 /*! \brief Hashing function for sorcery instances */
-AO2_STRING_FIELD_HASH_FN(ast_sorcery, module_name)
+AO2_STRING_FIELD_HASH_FN(sorcery_proxy, module_name)
 
 int ast_sorcery_init(void)
 {
@@ -392,7 +397,7 @@ int ast_sorcery_init(void)
 	}
 
 	instances = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, INSTANCE_BUCKETS,
-		ast_sorcery_hash_fn, NULL, ast_sorcery_cmp_fn);
+		sorcery_proxy_hash_fn, NULL, sorcery_proxy_cmp_fn);
 	if (!instances) {
 		return -1;
 	}
@@ -410,27 +415,6 @@ static void sorcery_internal_wizard_destructor(void *obj)
 }
 
 int __ast_sorcery_wizard_register(const struct ast_sorcery_wizard *interface, struct ast_module *module)
-{
-	struct ast_sorcery_wizard compat = {
-		.name = interface->name,
-		.open = interface->open,
-		.load = interface->load,
-		.reload = interface->reload,
-		.create = interface->create,
-		.retrieve_id = interface->retrieve_id,
-		.retrieve_regex = interface->retrieve_regex,
-		.retrieve_fields = interface->retrieve_fields,
-		.retrieve_multiple = interface->retrieve_multiple,
-		.update = interface->update,
-		.delete = interface->delete,
-		.close = interface->close,
-		.retrieve_prefix = NULL,
-	};
-
-	return __ast_sorcery_wizard_register_with_prefix(&compat, module);
-}
-
-int __ast_sorcery_wizard_register_with_prefix(const struct ast_sorcery_wizard *interface, struct ast_module *module)
 {
 	struct ast_sorcery_internal_wizard *wizard;
 	int res = -1;
@@ -592,61 +576,87 @@ static void sorcery_destructor(void *obj)
 
 /*! \brief Hashing function for sorcery types */
 AO2_STRING_FIELD_HASH_FN(ast_sorcery_object_type, name)
-
-/*! \brief Comparator function for sorcery types */
 AO2_STRING_FIELD_CMP_FN(ast_sorcery_object_type, name)
 
-struct ast_sorcery *__ast_sorcery_open(const char *module_name)
+static void sorcery_proxy_cb(void *weakproxy, void *data)
 {
+	ao2_unlink(instances, weakproxy);
+}
+
+struct ast_sorcery *__ast_sorcery_open(const char *module_name, const char *file, int line, const char *func)
+{
+	struct sorcery_proxy *proxy;
 	struct ast_sorcery *sorcery;
 
 	ast_assert(module_name != NULL);
 
 	ao2_wrlock(instances);
-	if ((sorcery = ao2_find(instances, module_name, OBJ_SEARCH_KEY | OBJ_NOLOCK))) {
-		goto done;
+	sorcery = __ao2_weakproxy_find(instances, module_name, OBJ_SEARCH_KEY | OBJ_NOLOCK,
+		__PRETTY_FUNCTION__, file, line, func);
+	if (sorcery) {
+		ao2_unlock(instances);
+
+		return sorcery;
 	}
 
-	if (!(sorcery = ao2_alloc(sizeof(*sorcery) + strlen(module_name) + 1, sorcery_destructor))) {
-		goto done;
+	proxy = ao2_t_weakproxy_alloc(sizeof(*proxy) + strlen(module_name) + 1, NULL, module_name);
+	if (!proxy) {
+		goto failure_cleanup;
+	}
+	strcpy(proxy->module_name, module_name); /* Safe */
+
+	sorcery = __ao2_alloc(sizeof(*sorcery), sorcery_destructor, AO2_ALLOC_OPT_LOCK_MUTEX, module_name, file, line, func);
+	if (!sorcery) {
+		goto failure_cleanup;
 	}
 
-	sorcery->types = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK, TYPE_BUCKETS,
-			ast_sorcery_object_type_hash_fn, ast_sorcery_object_type_cmp_fn);
+	sorcery->module_name = proxy->module_name;
+
+	/* We have exclusive access to proxy and sorcery, no need for locking here. */
+	if (ao2_t_weakproxy_set_object(proxy, sorcery, OBJ_NOLOCK, "weakproxy link")) {
+		goto failure_cleanup;
+	}
+
+	if (ao2_weakproxy_subscribe(proxy, sorcery_proxy_cb, NULL, OBJ_NOLOCK)) {
+		goto failure_cleanup;
+	}
+
+	sorcery->types = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, TYPE_BUCKETS,
+		ast_sorcery_object_type_hash_fn, NULL, ast_sorcery_object_type_cmp_fn);
 	if (!sorcery->types) {
-		ao2_ref(sorcery, -1);
-		sorcery = NULL;
-		goto done;
+		goto failure_cleanup;
 	}
 
 	if (!(sorcery->observers = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, NULL, NULL))) {
-		ao2_ref(sorcery, -1);
-		sorcery = NULL;
-		goto done;
+		goto failure_cleanup;
 	}
 
-	strcpy(sorcery->module_name, module_name); /* Safe */
-
 	if (__ast_sorcery_apply_config(sorcery, module_name, module_name) == AST_SORCERY_APPLY_FAIL) {
 		ast_log(LOG_ERROR, "Error attempting to apply configuration %s to sorcery.\n", module_name);
-		ao2_cleanup(sorcery);
-		sorcery = NULL;
-		goto done;
+		goto failure_cleanup;
 	}
 
-	ao2_link_flags(instances, sorcery, OBJ_NOLOCK);
+	ao2_link_flags(instances, proxy, OBJ_NOLOCK);
+	ao2_ref(proxy, -1);
 
 	NOTIFY_GLOBAL_OBSERVERS(observers, instance_created, module_name, sorcery);
 
-done:
 	ao2_unlock(instances);
 	return sorcery;
+
+failure_cleanup:
+	/* cleanup of sorcery may result in locking instances, so make sure we unlock first. */
+	ao2_unlock(instances);
+	ao2_cleanup(sorcery);
+	ao2_cleanup(proxy);
+
+	return NULL;
 }
 
 /*! \brief Search function for sorcery instances */
 struct ast_sorcery *ast_sorcery_retrieve_by_module_name(const char *module_name)
 {
-	return ao2_find(instances, module_name, OBJ_SEARCH_KEY);
+	return ao2_weakproxy_find(instances, module_name, OBJ_SEARCH_KEY, "");
 }
 
 /*! \brief Destructor function for object types */
@@ -822,16 +832,19 @@ enum ast_sorcery_apply_result __ast_sorcery_insert_wizard_mapping(struct ast_sor
 	RAII_VAR(struct ast_sorcery_object_wizard *, object_wizard, ao2_alloc(sizeof(*object_wizard), sorcery_object_wizard_destructor), ao2_cleanup);
 	int created = 0;
 
-	if (!wizard) {
+	if (!object_wizard) {
+		return AST_SORCERY_APPLY_FAIL;
+	}
+
+	if (!wizard || wizard->callbacks.module != ast_module_running_ref(wizard->callbacks.module)) {
 		ast_log(LOG_ERROR, "Wizard '%s' could not be applied to object type '%s' as it was not found\n",
 			name, type);
 		return AST_SORCERY_APPLY_FAIL;
-	} else if (!object_wizard) {
-		return AST_SORCERY_APPLY_FAIL;
 	}
 
 	if (!object_type) {
 		if (!(object_type = sorcery_object_type_alloc(type, module))) {
+			ast_module_unref(wizard->callbacks.module);
 			return AST_SORCERY_APPLY_FAIL;
 		}
 		created = 1;
@@ -848,6 +861,7 @@ enum ast_sorcery_apply_result __ast_sorcery_insert_wizard_mapping(struct ast_sor
 			ast_debug(1, "Wizard %s already applied to object type %s\n",
 					wizard->callbacks.name, object_type->name);
 			AST_VECTOR_RW_UNLOCK(&object_type->wizards);
+			ast_module_unref(wizard->callbacks.module);
 			return AST_SORCERY_APPLY_DUPLICATE;
 		}
 	}
@@ -858,11 +872,10 @@ enum ast_sorcery_apply_result __ast_sorcery_insert_wizard_mapping(struct ast_sor
 		ast_log(LOG_WARNING, "Wizard '%s' failed to open mapping for object type '%s' with data: %s\n",
 			name, object_type->name, S_OR(data, ""));
 		AST_VECTOR_RW_UNLOCK(&object_type->wizards);
+		ast_module_unref(wizard->callbacks.module);
 		return AST_SORCERY_APPLY_FAIL;
 	}
 
-	ast_module_ref(wizard->callbacks.module);
-
 	object_wizard->wizard = ao2_bump(wizard);
 	object_wizard->caching = caching;
 
@@ -1581,9 +1594,26 @@ static void sorcery_object_destructor(void *object)
 	ast_free(details->object->id);
 }
 
+void *ast_sorcery_lockable_alloc(size_t size, ao2_destructor_fn destructor, void *lockobj)
+{
+	void *object = ao2_alloc_with_lockobj(size + sizeof(struct ast_sorcery_object),
+		sorcery_object_destructor, lockobj, "");
+	struct ast_sorcery_object_details *details = object;
+
+	if (!object) {
+		return NULL;
+	}
+
+	details->object = object + size;
+	details->object->destructor = destructor;
+
+	return object;
+}
+
 void *ast_sorcery_generic_alloc(size_t size, ao2_destructor_fn destructor)
 {
-	void *object = ao2_alloc_options(size + sizeof(struct ast_sorcery_object), sorcery_object_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	void *object = ao2_alloc_options(size + sizeof(struct ast_sorcery_object),
+		sorcery_object_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK);
 	struct ast_sorcery_object_details *details = object;
 
 	if (!object) {
@@ -1619,6 +1649,7 @@ void *ast_sorcery_alloc(const struct ast_sorcery *sorcery, const char *type, con
 		return NULL;
 	}
 
+	details->object->created = ast_tvnow();
 	ast_copy_string(details->object->type, type, sizeof(details->object->type));
 
 	if (aco_set_defaults(&object_type->type, id, details)) {
@@ -1761,7 +1792,8 @@ void *ast_sorcery_retrieve_by_fields(const struct ast_sorcery *sorcery, const ch
 
 	/* If returning multiple objects create a container to store them in */
 	if ((flags & AST_RETRIEVE_FLAG_MULTIPLE)) {
-		if (!(object = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL))) {
+		object = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
+		if (!object) {
 			return NULL;
 		}
 	}
@@ -1805,7 +1837,12 @@ struct ao2_container *ast_sorcery_retrieve_by_regex(const struct ast_sorcery *so
 	struct ao2_container *objects;
 	int i;
 
-	if (!object_type || !(objects = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL))) {
+	if (!object_type) {
+		return NULL;
+	}
+
+	objects = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
+	if (!objects) {
 		return NULL;
 	}
 
@@ -1835,7 +1872,12 @@ struct ao2_container *ast_sorcery_retrieve_by_prefix(const struct ast_sorcery *s
 	struct ao2_container *objects;
 	int i;
 
-	if (!object_type || !(objects = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL))) {
+	if (!object_type) {
+		return NULL;
+	}
+
+	objects = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
+	if (!objects) {
 		return NULL;
 	}
 
@@ -2123,16 +2165,33 @@ int ast_sorcery_delete(const struct ast_sorcery *sorcery, void *object)
 	return object_wizard ? 0 : -1;
 }
 
-void ast_sorcery_unref(struct ast_sorcery *sorcery)
+int ast_sorcery_is_stale(const struct ast_sorcery *sorcery, void *object)
 {
-	if (sorcery) {
-		/* One ref for what we just released, the other for the instances container. */
-		ao2_wrlock(instances);
-		if (ao2_ref(sorcery, -1) == 2) {
-			ao2_unlink_flags(instances, sorcery, OBJ_NOLOCK);
-		}
-		ao2_unlock(instances);
+	const struct ast_sorcery_object_details *details = object;
+	RAII_VAR(struct ast_sorcery_object_type *, object_type, ao2_find(sorcery->types, details->object->type, OBJ_KEY), ao2_cleanup);
+	struct ast_sorcery_object_wizard *found_wizard;
+	int res = 0;
+	int i;
+
+	if (!object_type) {
+		return -1;
 	}
+
+	AST_VECTOR_RW_RDLOCK(&object_type->wizards);
+	for (i = 0; i < AST_VECTOR_SIZE(&object_type->wizards); i++) {
+		found_wizard = AST_VECTOR_GET(&object_type->wizards, i);
+
+		if (found_wizard->wizard->callbacks.is_stale) {
+			res |= found_wizard->wizard->callbacks.is_stale(sorcery, found_wizard->data, object);
+			ast_debug(5, "After calling wizard '%s', object '%s' is %s\n",
+				found_wizard->wizard->callbacks.name,
+				ast_sorcery_object_get_id(object),
+				res ? "stale" : "not stale");
+		}
+	}
+	AST_VECTOR_RW_UNLOCK(&object_type->wizards);
+
+	return res;
 }
 
 const char *ast_sorcery_object_get_id(const void *object)
@@ -2141,6 +2200,12 @@ const char *ast_sorcery_object_get_id(const void *object)
 	return details->object->id;
 }
 
+const struct timeval ast_sorcery_object_get_created(const void *object)
+{
+	const struct ast_sorcery_object_details *details = object;
+	return details->object->created;
+}
+
 const char *ast_sorcery_object_get_type(const void *object)
 {
 	const struct ast_sorcery_object_details *details = object;
