@@ -984,7 +984,7 @@ struct stasis_caching_topic *stasis_caching_topic_create(struct stasis_topic *or
 	}
 	ast_free(new_name);
 
-	caching_topic->sub = internal_stasis_subscribe(original_topic, caching_topic_exec, caching_topic, 0, 0);
+	caching_topic->sub = internal_stasis_subscribe(original_topic, caching_topic_exec, caching_topic, 0, 0, __FILE__, __LINE__, __PRETTY_FUNCTION__);
 	if (caching_topic->sub == NULL) {
 		ao2_ref(caching_topic, -1);
 
