@@ -2730,9 +2730,17 @@ int __ast_fd_set_flags(int fd, int flags, enum ast_fd_flag_operation op,
 
 	switch (op) {
 	case AST_FD_FLAG_SET:
+		if ((f & flags) == flags) {
+			/* There is nothing to set */
+			return 0;
+		}
 		f |= flags;
 		break;
 	case AST_FD_FLAG_CLEAR:
+		if (!(f & flags)) {
+			/* There is nothing to clear */
+			return 0;
+		}
 		f &= ~flags;
 		break;
 	default:
@@ -2750,6 +2758,42 @@ int __ast_fd_set_flags(int fd, int flags, enum ast_fd_flag_operation op,
 	return 0;
 }
 
+#ifndef HAVE_SOCK_NONBLOCK
+int ast_socket_nonblock(int domain, int type, int protocol)
+{
+	int s = socket(domain, type, protocol);
+	if (s < 0) {
+		return -1;
+	}
+
+	if (ast_fd_set_flags(s, O_NONBLOCK)) {
+		close(s);
+		return -1;
+	}
+
+	return s;
+}
+#endif
+
+#ifndef HAVE_PIPE2
+int ast_pipe_nonblock(int filedes[2])
+{
+	int p = pipe(filedes);
+	if (p < 0) {
+		return -1;
+	}
+
+	if (ast_fd_set_flags(filedes[0], O_NONBLOCK)
+	   || ast_fd_set_flags(filedes[1], O_NONBLOCK)) {
+		close(filedes[0]);
+		close(filedes[1]);
+		return -1;
+	}
+
+	return 0;
+}
+#endif
+
 /*!
  * \brief A thread local indicating whether the current thread is a user interface.
  */
