@@ -82,8 +82,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 
 #include "asterisk/paths.h"	/* ast_config_AST_CONFIG_DIR */
@@ -189,29 +187,49 @@ static char *handle_cli_dialplan_remove_include(struct ast_cli_entry *e, int cmd
 /*! \brief return true if 'name' is included by context c */
 static int lookup_ci(struct ast_context *c, const char *name)
 {
-	struct ast_include *i = NULL;
+	int idx;
+	int ret = 0;
 
-	if (ast_rdlock_context(c))	/* error, skip */
+	if (ast_rdlock_context(c)) {
+		/* error, skip */
 		return 0;
-	while ( (i = ast_walk_context_includes(c, i)) )
-		if (!strcmp(name, ast_get_include_name(i)))
+	}
+
+	for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+		const struct ast_include *i = ast_context_includes_get(c, idx);
+
+		if (!strcmp(name, ast_get_include_name(i))) {
+			ret = -1;
 			break;
+		}
+	}
 	ast_unlock_context(c);
-	return i ? -1 /* success */ : 0;
+
+	return ret;
 }
 
 /*! \brief return true if 'name' is in the ignorepats for context c */
 static int lookup_c_ip(struct ast_context *c, const char *name)
 {
-	struct ast_ignorepat *ip = NULL;
+	int idx;
+	int ret = 0;
 
-	if (ast_rdlock_context(c))	/* error, skip */
+	if (ast_rdlock_context(c)) {
+		/* error, skip */
 		return 0;
-	while ( (ip = ast_walk_context_ignorepats(c, ip)) )
-		if (!strcmp(name, ast_get_ignorepat_name(ip)))
+	}
+
+	for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+		const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
+
+		if (!strcmp(name, ast_get_ignorepat_name(ip))) {
+			ret = -1;
 			break;
+		}
+	}
 	ast_unlock_context(c);
-	return ip ? -1 /* success */ : 0;
+
+	return ret;
 }
 
 /*! \brief moves to the n-th word in the string, or empty string if none */
@@ -253,7 +271,7 @@ static int split_ec(const char *src, char **ext, char ** const ctx, char ** cons
 		*c++ = '\0';
 		*ctx = c;
 		if (strchr(c, '@')) { /* two @, not allowed */
-			free(e);
+			ast_free(e);
 			return -1;
 		}
 	}
@@ -282,12 +300,13 @@ static char *complete_dialplan_remove_include(struct ast_cli_args *a)
 		}
 		/* walk contexts and their includes, return the n-th match */
 		while (!res && (c = ast_walk_contexts(c))) {
-			struct ast_include *i = NULL;
+			int idx;
 
 			if (ast_rdlock_context(c))	/* error ? skip this one */
 				continue;
 
-			while ( !res && (i = ast_walk_context_includes(c, i)) ) {
+			for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+				const struct ast_include *i = ast_context_includes_get(c, idx);
 				const char *i_name = ast_get_include_name(i);
 				struct ast_context *nc = NULL;
 				int already_served = 0;
@@ -304,7 +323,7 @@ static char *complete_dialplan_remove_include(struct ast_cli_args *a)
 					already_served = lookup_ci(nc, i_name);
 
 				if (!already_served && ++which > a->n) {
-					res = strdup(i_name);
+					res = ast_strdup(i_name);
 					break;
 				}
 			}
@@ -323,7 +342,7 @@ static char *complete_dialplan_remove_include(struct ast_cli_args *a)
 
 		if (a->n > 0)
 			return NULL;
-		context = dupline = strdup(s);
+		context = dupline = ast_strdup(s);
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Out of free memory\n");
 			return NULL;
@@ -332,18 +351,18 @@ static char *complete_dialplan_remove_include(struct ast_cli_args *a)
 
 		if (ast_rdlock_contexts()) {
 			ast_log(LOG_ERROR, "Failed to lock contexts list\n");
-			free(context);
+			ast_free(context);
 			return NULL;
 		}
 
 		/* go through all contexts and check if is included ... */
 		while (!res && (c = ast_walk_contexts(c)))
 			if (lookup_ci(c, context)) /* context is really included, complete "from" command */
-				res = strdup("from");
+				res = ast_strdup("from");
 		ast_unlock_contexts();
 		if (!res)
 			ast_log(LOG_WARNING, "%s not included anywhere\n", context);
-		free(context);
+		ast_free(context);
 		return res;
 	} else if (a->pos == 5) { /* "dialplan remove include CTX from _X_" */
 		/*
@@ -351,7 +370,7 @@ static char *complete_dialplan_remove_include(struct ast_cli_args *a)
 		 */
 		char *context, *dupline, *from;
 		const char *s = skip_words(a->line, 3); /* skip 'dialplan' 'remove' 'include' */
-		context = dupline = strdup(s);
+		context = dupline = ast_strdup(s);
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Out of free memory\n");
 			return NULL;
@@ -362,13 +381,13 @@ static char *complete_dialplan_remove_include(struct ast_cli_args *a)
 		/* fourth word must be 'from' */
 		from = strsep(&dupline, " ");
 		if (!from || strcmp(from, "from")) {
-			free(context);
+			ast_free(context);
 			return NULL;
 		}
 
 		if (ast_rdlock_contexts()) {
 			ast_log(LOG_ERROR, "Failed to lock context list\n");
-			free(context);
+			ast_free(context);
 			return NULL;
 		}
 
@@ -380,10 +399,10 @@ static char *complete_dialplan_remove_include(struct ast_cli_args *a)
 				continue;
 			/* walk through all includes and check if it is our context */
 			if (lookup_ci(c, context) && ++which > a->n)
-				res = strdup(c_name);
+				res = ast_strdup(c_name);
 		}
 		ast_unlock_contexts();
-		free(context);
+		ast_free(context);
 		return res;
 	}
 
@@ -453,7 +472,7 @@ static char *handle_cli_dialplan_remove_extension(struct ast_cli_entry *e, int c
 	if ((!strlen(exten)) || (!(strlen(context)))) {
 		ast_cli(a->fd, "Missing extension or context name in third argument '%s'\n",
 			a->argv[3]);
-		free(exten);
+		ast_free(exten);
 		return CLI_FAILURE;
 	}
 
@@ -476,7 +495,7 @@ static char *handle_cli_dialplan_remove_extension(struct ast_cli_entry *e, int c
 		}
 		ret = CLI_FAILURE;
 	}
-	free(exten);
+	ast_free(exten);
 	return ret;
 }
 
@@ -588,7 +607,7 @@ static char *complete_dialplan_remove_extension(struct ast_cli_args *a)
 
 		ast_unlock_contexts();
 	error2:
-		free(exten);
+		ast_free(exten);
 	} else if (a->pos == 4) { /* 'dialplan remove extension EXT _X_' (priority) */
 		char *exten = NULL, *context, *cid, *p;
 		struct ast_context *c;
@@ -636,7 +655,7 @@ static char *complete_dialplan_remove_extension(struct ast_cli_args *a)
 				while ( !ret && (priority = ast_walk_extension_priorities(e, priority)) ) {
 					snprintf(buffer, sizeof(buffer), "%d", ast_get_extension_priority(priority));
 					if (partial_match(buffer, a->word, len) && ++which > a->n) /* n-th match */
-						ret = strdup(buffer);
+						ret = ast_strdup(buffer);
 				}
 				break;
 			}
@@ -644,7 +663,7 @@ static char *complete_dialplan_remove_extension(struct ast_cli_args *a)
 		}
 		ast_unlock_contexts();
 	error3:
-		free(exten);
+		ast_free(exten);
 	}
 	return ret;
 }
@@ -736,16 +755,16 @@ static char *complete_dialplan_add_include(struct ast_cli_args *a)
 		}
 		for (c = NULL; !ret && (c = ast_walk_contexts(c)); )
 			if (partial_match(ast_get_context_name(c), a->word, len) && ++which > a->n)
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 		ast_unlock_contexts();
 		return ret;
 	} else if (a->pos == 4) { /* dialplan add include CTX _X_ */
 		/* always complete  as 'into' */
-		return (a->n == 0) ? strdup("into") : NULL;
+		return (a->n == 0) ? ast_strdup("into") : NULL;
 	} else if (a->pos == 5) { /* 'dialplan add include CTX into _X_' (dst context) */
 		char *context, *dupline, *into;
 		const char *s = skip_words(a->line, 3); /* should not fail */
-		context = dupline = strdup(s);
+		context = dupline = ast_strdup(s);
 
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Out of free memory\n");
@@ -772,12 +791,12 @@ static char *complete_dialplan_add_include(struct ast_cli_args *a)
 			if (partial_match(ast_get_context_name(c), a->word, len) &&
 					!lookup_ci(c, context) /* not included yet */ &&
 					++which > a->n) {
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 			}
 		}
 		ast_unlock_contexts();
 	error3:
-		free(context);
+		ast_free(context);
 		return ret;
 	}
 
@@ -908,9 +927,7 @@ static char *handle_cli_dialplan_save(struct ast_cli_entry *e, int cmd, struct a
 	for (c = NULL; (c = ast_walk_contexts(c)); ) {
 		int context_header_written = 0;
 		struct ast_exten *ext, *last_written_e = NULL;
-		struct ast_include *i;
-		struct ast_ignorepat *ip;
-		struct ast_sw *sw;
+		int idx;
 
 		/* try to lock context and fireout all info */
 		if (ast_rdlock_context(c)) { /* lock failure */
@@ -985,17 +1002,22 @@ static char *handle_cli_dialplan_save(struct ast_cli_entry *e, int cmd, struct a
 			fprintf(output, "\n");
 
 		/* walk through includes */
-		for (i = NULL; (i = ast_walk_context_includes(c, i)) ; ) {
+		for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+			const struct ast_include *i = ast_context_includes_get(c, idx);
+
 			if (strcmp(ast_get_include_registrar(i), registrar) != 0)
 				continue; /* not mine */
 			PUT_CTX_HDR;
 			fprintf(output, "include => %s\n", ast_get_include_name(i));
 		}
-		if (ast_walk_context_includes(c, NULL))
+		if (ast_context_includes_count(c)) {
 			fprintf(output, "\n");
+		}
 
 		/* walk through switches */
-		for (sw = NULL; (sw = ast_walk_context_switches(c, sw)) ; ) {
+		for (idx = 0; idx < ast_context_switches_count(c); idx++) {
+			const struct ast_sw *sw = ast_context_switches_get(c, idx);
+
 			if (strcmp(ast_get_switch_registrar(sw), registrar) != 0)
 				continue; /* not mine */
 			PUT_CTX_HDR;
@@ -1003,11 +1025,14 @@ static char *handle_cli_dialplan_save(struct ast_cli_entry *e, int cmd, struct a
 				    ast_get_switch_name(sw), ast_get_switch_data(sw));
 		}
 
-		if (ast_walk_context_switches(c, NULL))
+		if (ast_context_switches_count(c)) {
 			fprintf(output, "\n");
+		}
 
 		/* fireout ignorepats ... */
-		for (ip = NULL; (ip = ast_walk_context_ignorepats(c, ip)); ) {
+		for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+			const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
+
 			if (strcmp(ast_get_ignorepat_registrar(ip), registrar) != 0)
 				continue; /* not mine */
 			PUT_CTX_HDR;
@@ -1217,7 +1242,7 @@ static int manager_dialplan_extension_add(struct mansession *s, const struct mes
 	}
 
 	if (ast_add_extension2(add_context, replace, exten, ipriority, NULL, cidmatch,
-			application, ast_strdup(application_data), ast_free_ptr, registrar)) {
+			application, ast_strdup(application_data), ast_free_ptr, registrar, NULL, 0)) {
 		ast_unlock_contexts();
 		switch (errno) {
 		case ENOMEM:
@@ -1270,7 +1295,7 @@ static char *complete_dialplan_remove_context(struct ast_cli_args *a)
 	/* walk through all contexts */
 	while ( !res && (c = ast_walk_contexts(c)) ) {
 		if (partial_match(ast_get_context_name(c), a->word, len) && ++which > a->n) {
-			res = strdup(ast_get_context_name(c));
+			res = ast_strdup(ast_get_context_name(c));
 		}
 	}
 	ast_unlock_contexts();
@@ -1283,7 +1308,7 @@ static char *complete_dialplan_add_extension(struct ast_cli_args *a)
 	int which = 0;
 
 	if (a->pos == 4) {		/* complete 'into' word ... */
-		return (a->n == 0) ? strdup("into") : NULL;
+		return (a->n == 0) ? ast_strdup("into") : NULL;
 	} else if (a->pos == 5) { /* complete context */
 		struct ast_context *c = NULL;
 		int len = strlen(a->word);
@@ -1298,11 +1323,11 @@ static char *complete_dialplan_add_extension(struct ast_cli_args *a)
 		/* walk through all contexts */
 		while ( !res && (c = ast_walk_contexts(c)) )
 			if (partial_match(ast_get_context_name(c), a->word, len) && ++which > a->n)
-				res = strdup(ast_get_context_name(c));
+				res = ast_strdup(ast_get_context_name(c));
 		ast_unlock_contexts();
 		return res;
 	} else if (a->pos == 6) {
-		return a->n == 0 ? strdup("replace") : NULL;
+		return a->n == 0 ? ast_strdup("replace") : NULL;
 	}
 	return NULL;
 }
@@ -1367,7 +1392,7 @@ static char *handle_cli_dialplan_add_ignorepat(struct ast_cli_entry *e, int cmd,
 static char *complete_dialplan_add_ignorepat(struct ast_cli_args *a)
 {
 	if (a->pos == 4)
-		return a->n == 0 ? strdup("into") : NULL;
+		return a->n == 0 ? ast_strdup("into") : NULL;
 	else if (a->pos == 5) {
 		struct ast_context *c;
 		int which = 0;
@@ -1380,7 +1405,7 @@ static char *complete_dialplan_add_ignorepat(struct ast_cli_args *a)
 		s = skip_words(a->line, 3);
 		if (s == NULL)
 			return NULL;
-		dupline = strdup(s);
+		dupline = ast_strdup(s);
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Malloc failure\n");
 			return NULL;
@@ -1400,10 +1425,10 @@ static char *complete_dialplan_add_ignorepat(struct ast_cli_args *a)
 			if (ignorepat) /* there must be one, right ? */
 				found = lookup_c_ip(c, ignorepat);
 			if (!found && ++which > a->n)
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 		}
 
-		free(ignorepat);
+		ast_free(ignorepat);
 		ast_unlock_contexts();
 		return ret;
 	}
@@ -1474,12 +1499,13 @@ static char *complete_dialplan_remove_ignorepat(struct ast_cli_args *a)
 		}
 
 		for (c = NULL; !ret && (c = ast_walk_contexts(c));) {
-			struct ast_ignorepat *ip;
+			int idx;
 
 			if (ast_rdlock_context(c))	/* error, skip it */
 				continue;
+			for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+				const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
 
-			for (ip = NULL; !ret && (ip = ast_walk_context_ignorepats(c, ip));) {
 				if (partial_match(ast_get_ignorepat_name(ip), a->word, len) && ++which > a->n) {
 					/* n-th match */
 					struct ast_context *cw = NULL;
@@ -1489,7 +1515,7 @@ static char *complete_dialplan_remove_ignorepat(struct ast_cli_args *a)
 						found = lookup_c_ip(cw, ast_get_ignorepat_name(ip));
 					}
 					if (!found)
-						ret = strdup(ast_get_ignorepat_name(ip));
+						ret = ast_strdup(ast_get_ignorepat_name(ip));
 				}
 			}
 			ast_unlock_context(c);
@@ -1497,12 +1523,12 @@ static char *complete_dialplan_remove_ignorepat(struct ast_cli_args *a)
 		ast_unlock_contexts();
 		return ret;
 	} else if (a->pos == 4) {
-		 return a->n == 0 ? strdup("from") : NULL;
+		 return a->n == 0 ? ast_strdup("from") : NULL;
 	} else if (a->pos == 5) { /* XXX check this */
 		char *dupline, *duplinet, *ignorepat;
 		int len = strlen(a->word);
 
-		dupline = strdup(a->line);
+		dupline = ast_strdup(a->line);
 		if (!dupline) {
 			ast_log(LOG_WARNING, "Out of free memory\n");
 			return NULL;
@@ -1514,13 +1540,13 @@ static char *complete_dialplan_remove_ignorepat(struct ast_cli_args *a)
 		ignorepat = strsep(&duplinet, " ");
 
 		if (!ignorepat) {
-			free(dupline);
+			ast_free(dupline);
 			return NULL;
 		}
 
 		if (ast_rdlock_contexts()) {
 			ast_log(LOG_WARNING, "Failed to lock contexts list\n");
-			free(dupline);
+			ast_free(dupline);
 			return NULL;
 		}
 
@@ -1534,11 +1560,11 @@ static char *complete_dialplan_remove_ignorepat(struct ast_cli_args *a)
 				continue;
 			}
 			if (lookup_c_ip(c, ignorepat) && ++which > a->n)
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 			ast_unlock_context(c);
 		}
 		ast_unlock_contexts();
-		free(dupline);
+		ast_free(dupline);
 		return ret;
 	}
 
@@ -1584,12 +1610,11 @@ static struct ast_cli_entry cli_pbx_config[] = {
 	AST_CLI_DEFINE(handle_cli_dialplan_remove_ignorepat, "Remove ignore pattern from context"),
 	AST_CLI_DEFINE(handle_cli_dialplan_add_include,      "Include context in other context"),
 	AST_CLI_DEFINE(handle_cli_dialplan_remove_include,   "Remove a specified include from context"),
-	AST_CLI_DEFINE(handle_cli_dialplan_reload,           "Reload extensions and *only* extensions"),
-	AST_CLI_DEFINE(handle_cli_dialplan_save,             "Save current dialplan into a file")
+	AST_CLI_DEFINE(handle_cli_dialplan_reload,           "Reload extensions and *only* extensions")
 };
 
 static struct ast_cli_entry cli_dialplan_save =
-	AST_CLI_DEFINE(handle_cli_dialplan_save, "Save dialplan");
+	AST_CLI_DEFINE(handle_cli_dialplan_save,             "Save current dialplan into a file");
 
 #define AMI_EXTENSION_ADD "DialplanExtensionAdd"
 #define AMI_EXTENSION_REMOVE "DialplanExtensionRemove"
@@ -1599,15 +1624,15 @@ static struct ast_cli_entry cli_dialplan_save =
  */
 static int unload_module(void)
 {
-	if (static_config && !write_protect_config)
-		ast_cli_unregister(&cli_dialplan_save);
-	if (overrideswitch_config) {
-		ast_free(overrideswitch_config);
-	}
+	ast_cli_unregister(&cli_dialplan_save);
+	ast_free(overrideswitch_config);
+	overrideswitch_config = NULL;
+
 	ast_cli_unregister_multiple(cli_pbx_config, ARRAY_LEN(cli_pbx_config));
 	ast_manager_unregister(AMI_EXTENSION_ADD);
 	ast_manager_unregister(AMI_EXTENSION_REMOVE);
 	ast_context_destroy(NULL, registrar);
+
 	return 0;
 }
 
@@ -1689,10 +1714,20 @@ static int pbx_load_config(const char *config_file)
 
 	ast_copy_string(userscontext, ast_variable_retrieve(cfg, "general", "userscontext") ?: "default", sizeof(userscontext));
 
-	for (v = ast_variable_browse(cfg, "globals"); v; v = v->next) {
-		pbx_substitute_variables_helper(NULL, v->value, realvalue, sizeof(realvalue) - 1);
-		pbx_builtin_setvar_helper(NULL, v->name, realvalue);
+	/* ast_variable_browse does not merge multiple [globals] sections */
+	for (cxt = ast_category_browse(cfg, NULL);
+	     cxt;
+	     cxt = ast_category_browse(cfg, cxt)) {
+		if (strcasecmp(cxt, "globals")) {
+			continue;
+		}
+
+		for (v = ast_variable_browse(cfg, cxt); v; v = v->next) {
+			pbx_substitute_variables_helper(NULL, v->value, realvalue, sizeof(realvalue) - 1);
+			pbx_builtin_setvar_helper(NULL, v->name, realvalue);
+		}
 	}
+
 	for (cxt = ast_category_browse(cfg, NULL);
 	     cxt;
 	     cxt = ast_category_browse(cfg, cxt)) {
@@ -1835,6 +1870,7 @@ process_extension:
 
 				appl = ast_skip_blanks(appl);
 				if (ipri) {
+					const char *registrar_file;
 					if (plus) {
 						ipri += atoi(plus);
 					}
@@ -1844,7 +1880,14 @@ process_extension:
 							"The use of '%s' for an extension is strongly discouraged and can have unexpected behavior.  Please use '_X%c' instead at line %d of %s\n",
 							realext, realext[1], v->lineno, vfile);
 					}
-					if (ast_add_extension2(con, 0, realext, ipri, label, cidmatch, appl, ast_strdup(data), ast_free_ptr, registrar)) {
+					/* Don't include full path if the configuration file includes slashes */
+					registrar_file = strrchr(vfile, '/');
+					if (!registrar_file) {
+						registrar_file = vfile;
+					} else {
+						registrar_file++; /* Skip past the end slash */
+					}
+					if (ast_add_extension2(con, 0, realext, ipri, label, cidmatch, appl, ast_strdup(data), ast_free_ptr, registrar, registrar_file, v->lineno)) {
 						ast_log(LOG_WARNING,
 							"Unable to register extension at line %d of %s\n",
 							v->lineno, vfile);
@@ -1906,6 +1949,8 @@ process_extension:
 						"Unable to include switch '%s' in context '%s' at line %d of %s\n",
 						v->value, cxt, v->lineno, vfile);
 				}
+			} else if (!strcasecmp(v->name, "autohints")) {
+				ast_context_set_autohints(con, ast_true(v->value));
 			} else {
 				ast_log(LOG_WARNING,
 					"==!!== Unknown directive: %s at line %d of %s -- IGNORING!!!\n",
@@ -2009,19 +2054,19 @@ static void pbx_load_users(void)
 			}
 
 			/* Add hint */
-			ast_add_extension2(con, 0, cat, -1, NULL, NULL, iface, NULL, NULL, registrar);
+			ast_add_extension2(con, 0, cat, -1, NULL, NULL, iface, NULL, NULL, registrar, NULL, 0);
 			/* If voicemail, use "stdexten" else use plain old dial */
 			if (hasvoicemail) {
 				if (ast_opt_stdexten_macro) {
 					/* Use legacy stdexten macro method. */
 					snprintf(tmp, sizeof(tmp), "stdexten,%s,${HINT}", cat);
-					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Macro", ast_strdup(tmp), ast_free_ptr, registrar);
+					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Macro", ast_strdup(tmp), ast_free_ptr, registrar, NULL, 0);
 				} else {
 					snprintf(tmp, sizeof(tmp), "%s,stdexten(${HINT})", cat);
-					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Gosub", ast_strdup(tmp), ast_free_ptr, registrar);
+					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Gosub", ast_strdup(tmp), ast_free_ptr, registrar, NULL, 0);
 				}
 			} else {
-				ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Dial", ast_strdup("${HINT}"), ast_free_ptr, registrar);
+				ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Dial", ast_strdup("${HINT}"), ast_free_ptr, registrar, NULL, 0);
 			}
 			altexts = ast_variable_retrieve(cfg, cat, "alternateexts");
 			if (!ast_strlen_zero(altexts)) {
@@ -2030,7 +2075,7 @@ static void pbx_load_users(void)
 				c = altcopy;
 				ext = strsep(&c, ",");
 				while (ext) {
-					ast_add_extension2(con, 0, ext, 1, NULL, NULL, "Goto", ast_strdup(tmp), ast_free_ptr, registrar);
+					ast_add_extension2(con, 0, ext, 1, NULL, NULL, "Goto", ast_strdup(tmp), ast_free_ptr, registrar, NULL, 0);
 					ext = strsep(&c, ",");
 				}
 			}
@@ -2045,10 +2090,17 @@ static int pbx_load_module(void)
 
 	ast_mutex_lock(&reload_lock);
 
-	if (!local_table)
+	if (!local_table) {
 		local_table = ast_hashtab_create(17, ast_hashtab_compare_contexts, ast_hashtab_resize_java, ast_hashtab_newsize_java, ast_hashtab_hash_contexts, 0);
+		if (!local_table) {
+			ast_mutex_unlock(&reload_lock);
+			return AST_MODULE_LOAD_DECLINE;
+		}
+	}
 
 	if (!pbx_load_config(config)) {
+		ast_hashtab_destroy(local_table, NULL);
+		local_table = NULL;
 		ast_mutex_unlock(&reload_lock);
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -2075,6 +2127,11 @@ static int load_module(void)
 {
 	int res;
 
+	if (pbx_load_module()) {
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
 	if (static_config && !write_protect_config)
 		ast_cli_register(&cli_dialplan_save);
 	ast_cli_register_multiple(cli_pbx_config, ARRAY_LEN(cli_pbx_config));
@@ -2089,9 +2146,6 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	if (pbx_load_module())
-		return AST_MODULE_LOAD_DECLINE;
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -2103,8 +2157,8 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Text Extension Configuration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
