@@ -24,15 +24,11 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_stasis_recording</depend>
-	<depend type="module">res_stasis_playback</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "resource_bridges.h"
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_bridges.h"
@@ -220,6 +216,12 @@ void ast_ari_bridges_add_channel(struct ast_variable *headers,
 				return;
 			}
 		}
+
+		/* Apply bridge features to each of the channel controls */
+		if (!stasis_app_control_bridge_features_init(list->controls[i])) {
+			stasis_app_control_absorb_dtmf_in_bridge(list->controls[i], args->absorb_dtmf);
+			stasis_app_control_mute_in_bridge(list->controls[i], args->mute);
+		}
 	}
 
 	for (i = 0; i < list->count; ++i) {
@@ -288,10 +290,9 @@ static void *bridge_channel_control_thread(void *data)
 	struct ast_channel *bridge_channel = thread_data->bridge_channel;
 	struct stasis_app_control *control = thread_data->control;
 	struct stasis_forward *forward = thread_data->forward;
+	ast_callid callid = ast_channel_callid(bridge_channel);
 	char *bridge_id = ast_strdupa(thread_data->bridge_id);
 
-	RAII_VAR(struct ast_callid *, callid, ast_channel_callid(bridge_channel), ast_callid_cleanup);
-
 	if (callid) {
 		ast_callid_threadassoc_add(callid);
 	}
@@ -337,7 +338,8 @@ static struct ast_channel *prepare_bridge_media_channel(const char *type)
  * \brief Performs common setup for a bridge playback operation
  * with both new controls and when existing controls are  found.
  *
- * \param args_media media string split from arguments
+ * \param args_media medias to play
+ * \param args_media_count number of media items in \c media
  * \param args_lang language string split from arguments
  * \param args_offset_ms milliseconds offset split from arguments
  * \param args_playback_id string to use for playback split from
@@ -351,7 +353,8 @@ static struct ast_channel *prepare_bridge_media_channel(const char *type)
  * \retval -1 operation failed
  * \retval operation was successful
  */
-static int ari_bridges_play_helper(const char *args_media,
+static int ari_bridges_play_helper(const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -376,8 +379,8 @@ static int ari_bridges_play_helper(const char *args_media,
 
 	language = S_OR(args_lang, snapshot->language);
 
-	playback = stasis_app_control_play_uri(control, args_media, language,
-		bridge->uniqueid, STASIS_PLAYBACK_TARGET_BRIDGE, args_skipms,
+	playback = stasis_app_control_play_uri(control, args_media, args_media_count,
+		language, bridge->uniqueid, STASIS_PLAYBACK_TARGET_BRIDGE, args_skipms,
 		args_offset_ms, args_playback_id);
 
 	if (!playback) {
@@ -400,7 +403,8 @@ static int ari_bridges_play_helper(const char *args_media,
 	return 0;
 }
 
-static void ari_bridges_play_new(const char *args_media,
+static void ari_bridges_play_new(const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -453,9 +457,9 @@ static void ari_bridges_play_new(const char *args_media,
 	}
 
 	ao2_lock(control);
-	if (ari_bridges_play_helper(args_media, args_lang, args_offset_ms,
-			args_skipms, args_playback_id, response, bridge, control,
-			&json, &playback_url)) {
+	if (ari_bridges_play_helper(args_media, args_media_count, args_lang,
+			args_offset_ms, args_skipms, args_playback_id, response, bridge,
+			control, &json, &playback_url)) {
 		ao2_unlock(control);
 		return;
 	}
@@ -505,7 +509,8 @@ enum play_found_result {
  * \brief Performs common setup for a bridge playback operation
  * with both new controls and when existing controls are  found.
  *
- * \param args_media media string split from arguments
+ * \param args_media medias to play
+ * \param args_media_count number of media items in \c media
  * \param args_lang language string split from arguments
  * \param args_offset_ms milliseconds offset split from arguments
  * \param args_playback_id string to use for playback split from
@@ -519,7 +524,8 @@ enum play_found_result {
  * \retval PLAY_FOUND_CHANNEL_UNAVAILABLE The operation failed because
  * the channel requested to playback with is breaking down.
  */
-static enum play_found_result ari_bridges_play_found(const char *args_media,
+static enum play_found_result ari_bridges_play_found(const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -545,9 +551,9 @@ static enum play_found_result ari_bridges_play_found(const char *args_media,
 		return PLAY_FOUND_CHANNEL_UNAVAILABLE;
 	}
 
-	if (ari_bridges_play_helper(args_media, args_lang, args_offset_ms,
-			args_skipms, args_playback_id, response, bridge, control,
-			&json, &playback_url)) {
+	if (ari_bridges_play_helper(args_media, args_media_count,
+			args_lang, args_offset_ms, args_skipms, args_playback_id,
+			response, bridge, control, &json, &playback_url)) {
 		ao2_unlock(control);
 		return PLAY_FOUND_FAILURE;
 	}
@@ -559,7 +565,8 @@ static enum play_found_result ari_bridges_play_found(const char *args_media,
 
 static void ari_bridges_handle_play(
 	const char *args_bridge_id,
-	const char *args_media,
+	const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -582,15 +589,15 @@ static void ari_bridges_handle_play(
 		 * that will work or else there isn't a channel for this bridge anymore,
 		 * in which case we'll revert to ari_bridges_play_new.
 		 */
-		if (ari_bridges_play_found(args_media, args_lang, args_offset_ms,
-				args_skipms, args_playback_id, response,bridge,
+		if (ari_bridges_play_found(args_media, args_media_count, args_lang,
+				args_offset_ms, args_skipms, args_playback_id, response,bridge,
 				play_channel) == PLAY_FOUND_CHANNEL_UNAVAILABLE) {
 			continue;
 		}
 		return;
 	}
 
-	ari_bridges_play_new(args_media, args_lang, args_offset_ms,
+	ari_bridges_play_new(args_media, args_media_count, args_lang, args_offset_ms,
 		args_skipms, args_playback_id, response, bridge);
 }
 
@@ -601,6 +608,7 @@ void ast_ari_bridges_play(struct ast_variable *headers,
 {
 	ari_bridges_handle_play(args->bridge_id,
 	args->media,
+	args->media_count,
 	args->lang,
 	args->offsetms,
 	args->skipms,
@@ -614,6 +622,7 @@ void ast_ari_bridges_play_with_id(struct ast_variable *headers,
 {
 	ari_bridges_handle_play(args->bridge_id,
 	args->media,
+	args->media_count,
 	args->lang,
 	args->offsetms,
 	args->skipms,
@@ -816,6 +825,7 @@ void ast_ari_bridges_start_moh(struct ast_variable *headers,
 	}
 
 	ast_moh_start(moh_channel, moh_class, NULL);
+	ast_channel_cleanup(moh_channel);
 
 	ast_ari_response_no_content(response);
 
