@@ -23,13 +23,11 @@
 /*** MODULEINFO
 	<depend>res_calendar</depend>
 	<depend>neon29</depend>
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ne_request.h>
 #include <ne_session.h>
 #include <ne_uri.h>
@@ -939,8 +937,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk MS Exchange Web Service Calendar Integration",
-	.support_level = AST_MODULE_SUPPORT_CORE,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
+	.requires = "res_calendar",
 );
