@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
@@ -160,8 +158,6 @@ static char *handle_orig(struct ast_cli_entry *e, int cmd, struct ast_cli_args *
 			"used. If no extension is given, the 's' extension will be used.\n";
 		return NULL;
 	case CLI_GENERATE:
-		/* ugly, can be removed when CLI entries have ast_module pointers */
-		ast_module_ref(ast_module_info->self);
 		if (a->pos == 3) {
 			res = ast_cli_complete(a->word, choices, a->n);
 		} else if (a->pos == 4) {
@@ -169,16 +165,12 @@ static char *handle_orig(struct ast_cli_entry *e, int cmd, struct ast_cli_args *
 				res = ast_complete_applications(a->line, a->word, a->n);
 			}
 		}
-		ast_module_unref(ast_module_info->self);
 		return res;
 	}
 
 	if (ast_strlen_zero(a->argv[2]) || ast_strlen_zero(a->argv[3]))
 		return CLI_SHOWUSAGE;
 
-	/* ugly, can be removed when CLI entries have ast_module pointers */
-	ast_module_ref(ast_module_info->self);
-
 	if (!strcasecmp("application", a->argv[3])) {
 		res = orig_app(a->fd, a->argv[2], a->argv[4], a->argv[5]);
 	} else if (!strcasecmp("extension", a->argv[3])) {
@@ -187,8 +179,6 @@ static char *handle_orig(struct ast_cli_entry *e, int cmd, struct ast_cli_args *
 		res = CLI_SHOWUSAGE;
 	}
 
-	ast_module_unref(ast_module_info->self);
-
 	return res;
 }
 
