@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include <corosync/cpg.h>
 #include <corosync/cfg.h>
 
@@ -1127,7 +1125,8 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	nodes = ao2_container_alloc(23, corosync_node_hash_fn, corosync_node_cmp_fn);
+	nodes = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 23,
+		corosync_node_hash_fn, NULL, corosync_node_cmp_fn);
 	if (!nodes) {
 		goto failed;
 	}
